// Import necessary D3 modules
import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  select,
  symbol,
  symbolCircle,
  symbolSquare,
  symbolStar,
  scaleOrdinal,
  schemeCategory10,
} from 'd3';

// Define the scatterPlot function
export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin = {
    top: 10,
    right: 10,
    bottom: 50,
    left: 100,
  }; // Adjusted margin for the y-axis label
  let symbolMapping = {
    'Research Environment': symbolStar,
    'Students to Staff Ratio': symbolSquare,
    'Research Quality': symbolCircle,
    'Industry Impact': symbolSquare,
  };

  const chart = (selection) => {
    selection.each(function () {
      // Create scales
      const xScale = scaleLinear()
        .domain(extent(data, (d) => d.Country)) // Use country names for xScale domain
        .range([0, width - margin.left - margin.right]);
      const yScale = scaleLinear()
        .domain(extent(data, yValue))
        .range([height - margin.top - margin.bottom, 0]);

      // Create the root container
      const svg = select(this)
        .selectAll('svg')
        .data([null]);
      const svgEnter = svg.enter().append('svg');
      svgEnter
        .merge(svg)
        .attr('width', width)
        .attr('height', height);

      // Append a 'g' element for chart
      const gEnter = svgEnter.append('g');
      const g = svg.select('g').merge(gEnter);
      g.attr(
        'transform',
        `translate(${margin.left},${margin.top})`,
      );

      // X-axis and the x-axis label
      const xAxis = axisBottom(xScale);
      const xAxisGEnter = gEnter
        .append('g')
        .attr('class', 'x-axis')
        .attr(
          'transform',
          `translate(0,${height - margin.top - margin.bottom})`,
        );
      xAxisGEnter
        .call(xAxis)
        .selectAll('text')
        .style('text-anchor', 'end')
        .attr('dx', '-.8em')
        .attr('dy', '.15em')
        .attr('transform', 'rotate(-65)');

      // Call the x-axis on update
      g.select('.x-axis').transition().call(xAxis);

      // Y-axis and the y-axis label
      const yAxis = axisLeft(yScale).ticks(10); // Adjust the number of ticks as needed
      const yAxisGEnter = gEnter
        .append('g')
        .attr('class', 'y-axis');
      yAxisGEnter
        .call(yAxis)
        .append('text')
        .attr('class', 'axis-label')
        .attr('transform', 'rotate(-90)')
        .attr('y', -70) // Adjust this to move the label closer to the y-axis line
        .attr(
          'x',
          -(height - margin.top - margin.bottom) / 2,
        )
        .attr('text-anchor', 'middle')
        .attr('fill', 'red')
        .text('Cancer Type'); // Text for the y-axis label

      // Call the y-axis on update
      g.select('.y-axis').transition().call(yAxis);

      // Draw symbols
      const symbols = g.selectAll('.symbol').data(data);

      symbols
        .enter()
        .append('path')
        .attr('class', 'symbol')
        .merge(symbols)
        .attr('d', (d) =>
          symbolMapping[d.type]() // Use symbol based on data type
            .size(64),
        )
        .attr(
          'transform',
          (d) =>
            `translate(${xScale(d.Country)},${yScale(yValue(d))})`,
        );
      symbols.exit().remove();
    });
  };

  // Setter functions for chart properties
  chart.width = (_) =>
    arguments.length ? ((width = +_), chart) : width;
  chart.height = (_) =>
    arguments.length ? ((height = +_), chart) : height;
  chart.data = (_) =>
    arguments.length ? ((data = _), chart) : data;
  chart.xValue = (_) =>
    arguments.length ? ((xValue = _), chart) : xValue;
  chart.yValue = (_) =>
    arguments.length ? ((yValue = _), chart) : yValue;
  chart.margin = (_) =>
    arguments.length ? ((margin = _), chart) : margin;

  return chart; // Return the chart function
};
